;
//文本抓取，并且过滤掉链接密度过高的区域，剩余链接密度低的区域参与进一步分析
(function(){
function getText() {
    var textResult = [];
    var tagBlackList = [];

    function addText(node, text) {
        if (text.length >= 25) {
            if (!isText(text)) {
                return;
            }
            textResult.push({"node":node, "text": text});
        }
    }

    function enumNode(node) {
        if (node == null) {
            return;
        }

        if (node.nodeType == Node.TEXT_NODE) {
            addText(node, node.textContent);
            return;
        }
        if (node.nodeType != Node.ELEMENT_NODE) {
            return;
        }

        var style = window.getComputedStyle(node);
        if (style.display == "none") {
            return;
        }

        if (tagBlackList.indexOf(node.tagName) >= 0) {
            return;
        }

        if (!allInlineChild(node)) {
            for (var i = 0; i < node.childNodes.length; i++) {
                var child = node.childNodes[i];
                enumNode(child);
            }
        } else {
            if (canClick(node)) {
                return;
            }
            var text = (node.innerText || "").trim();
            addText(node, text);
        }
    }
    enumNode(document.body);
    return removeComment(textResult);
}

//移除链接密度过高的区域
function removeComment(data) {
    var group = getParentNode(data);
    var toRemove = [];
    for (var i = 0; i < group.length; i++) {
        var p = group[i];
        var child = getChild(p, data);
        if (hasTooMuchLink(p, child)) {
            for (var j = 0; j < child.length; j++) {
                toRemove.push(child[j]);
            }
        }
    }

    var result = [];
    for (var i = 0; i < data.length; i++) {
        if (toRemove.indexOf(data[i].node) < 0) {
            result.push(data[i].text);
        }
    }
    return result;
}

function getChild(p, data) {
    var result = [];
    for (var i = 0; i < data.length; i++) {
        if (isChild(p, data[i].node)) {
            result.push(data[i].node);
        }
    }
    return result;
}

function isChild(parent, child) {
    var p = child.parentNode;
    while(p != null) {
        if (p == parent) {
            return true;
        }
        p = p.parentNode;
    }
    return false;
}

function getCommonParent(a, b) {
    if (a == b) {
        return a;
    }

    if (isChild(a, b)) {
        return a;
    }

    if (isChild(b, a)) {
        return b;
    }

    var p = a.parentNode;
    while(p != null) {
        if (isChild(p, b)) {
            return p;
        }
        p = p.parentNode;
    }

    return null;
}

//链接密度过高
function hasTooMuchLink(node, childNodes) {
    var elements = node.getElementsByTagName("*");
    var linkCount = 0;
    var start = false;
    for (var i = 0; i < elements.length; i++) {
        var el = elements[i];
        if (el == childNodes[0]) {
            start = true;
        }

        if (el == childNodes[childNodes.length - 1]) {
            break;
        }

        if (start && el.tagName == "A" && el.innerText) {
            var aTagParentNode=el.parentNode;
            if(aTagParentNode.innerText.length > 60 && allInlineChild(aTagParentNode)){
                continue;
            }
            linkCount++;
        }
    }
    var excludeNumber = 0;
    for (var i = 0; i < childNodes.length - 1; i++) {
        if (childNodes[i].nodeType == Node.ELEMENT_NODE) {
            excludeNumber += childNodes[i].getElementsByTagName("A").length;
        }
    }
    if ((linkCount - excludeNumber) / childNodes.length > 0.5) {
        return true;
    }
    return false;
}

function getParentNode(data) {
    var parents = [];
    for (var i = 0; i < data.length - 1; i++) {
        var p = getCommonParent(data[i].node, data[i + 1].node);
        if (parents.indexOf(p) < 0) {
            parents.push(p);
        }
    }

    var toRemove = [];
    for (var i = 0; i < parents.length; i++) {
        var current = parents[i];
        for (var j = 0; j < parents.length; j++) {
            if (i != j && isChild(current, parents[j])) {
                toRemove.push(current);
            }
        }
    }

    var result = [];
    for (var i = 0; i < parents.length; i++) {
        if (toRemove.indexOf(parents[i]) < 0) {
            result.push(parents[i]);
        }
    }
    return result;
}

//是否所有子节点都是inline
function allInlineChild(node) {
    var childs = node.getElementsByTagName("*");
    for (var i = 0; i < childs.length; i++) {
        var child = childs[i];
        var style = window.getComputedStyle(child);
        if (style.display != "inline") {
            return false;
        }
    }
    return true;
}

function isText(s) {
    var reg = /[\u4e00-\u9fa5a-zA-Z]/g //only check chinese and English currently
    var textlen = (s.match(reg) || []).length;
    return (textlen / s.length > 0.5) || (textlen > 15)
}

//节点是否可点击
function canClick(node) {
    function hasClickEvent(n){
        if (n.tagName === 'A' || n.tagName === 'BUTTON') {
            return true;
        }
    }

    if (hasClickEvent(node)) {
        return true;
    }

    var childs = node.getElementsByTagName("*");
    var totalSize = (node.innerText || "").replace(/[^\u4e00-\u9fa5a-zA-Z]/g, "").length;
    var linkSize = 0;
    for (var i = 0; i < childs.length; i++) {
        var child = childs[i];
        if (hasClickEvent(child)) {
            linkSize += (child.innerText || "").length;
        }
    }

    if (linkSize / totalSize > 0.3) {
        return true;
    }

    var p = node.parentNode;
    for (var i = 0; i < 3; i++) {
        if (p == null) {
            return false;
        }
        if (hasClickEvent(p)) {
            return true;
        }
        p = p.parentNode;
    }
    return false;
}

function checkText(text) {
    var totalSize = 0;
    var hasLongText = false;
    for (var i = 0; i < text.length; i++) {
        if (text[i].length > 60) {
            hasLongText = true;
        }
        totalSize += text[i].length;
    }
    return (totalSize > 180) && hasLongText;
}

//检查是否有超过两张的大图
function getImgCount() {
    function validImg(img) {
        var style = window.getComputedStyle(img);
        if (style && style.display && style.display != "none")  {
            var width = img.clientWidth;
            var height = img.clientHeight;
            if (width / height > 2.9) {
                return false;
            }
            if (width < minImgWidth) {
                return false;
            }

            var rect = img.getBoundingClientRect();
            if (rect.left >= screenWidth || rect.right <= 0) {
                return false
            }

            return true;
        }
        return false;
    }

    var screenWidth = document.body.clientWidth;
    var minImgWidth = Math.min(screenWidth * (2 / 3), 400);
    var img = document.getElementsByTagName("IMG");
    var bigImgCount = 0;
    for (var i = 0; i < img.length; i++) {
        if (validImg(img[i])) {
            bigImgCount++;
        }
    }

    if (packageName == "com.ifeng.news2") { // for ifeng news, images a background-image of a div
        var img = document.getElementsByTagName("DIV");
        for (var i = 0; i < img.length; i++) {
            var background = window.getComputedStyle(img[i])["background-image"];
            if (background && background != "none" && validImg(img[i]) && img[i].innerText == "") {
                bigImgCount++;
            }
        }
    }
    LbDebug("bigImgCount " + bigImgCount);
    return bigImgCount;
}

function hasIframe() {
    function bigEnough(frame) {
        var rect = frame.getBoundingClientRect();
        if (rect.width > minIFrameWidth && rect.width / rect.height <= 2.9) {
            return true
        }
        return false;
    }

    var screenWidth = document.body.clientWidth;
    var minIFrameWidth = Math.min(screenWidth * (2 / 3), 400);
    var iframe = document.getElementsByTagName("IFRAME");
    for (var i = 0; i < iframe.length; i++) {
        if (bigEnough(iframe[i])) {
            return true
        }
    }
    return false;
}

function hasLongImg() {
    function validImg(img) {
        var style = window.getComputedStyle(img);
        if (style && style.display && style.display != "none")  {
            var width = img.clientWidth;
            var height = img.clientHeight;
            if (width > minImgWidth && height / width > 2.9) {
                return true;
            }
        }
        return false;
    }

    var screenWidth = document.body.clientWidth;
    var minImgWidth = Math.min(screenWidth * (2 / 3), 400);
    var img = document.getElementsByTagName("IMG");
    var hasLong = false;
    for (var i = 0; i < img.length; i++) {
        if (validImg(img[i])) {
            hasLong = true;
            break;
        }
    }

    if (packageName == "com.ifeng.news2") { // for ifeng news, images a background-image of a div
        var img = document.getElementsByTagName("DIV");
        for (var i = 0; i < img.length; i++) {
            var background = window.getComputedStyle(img[i])["background-image"];
            if (background && background != "none" && validImg(img[i]) && img[i].innerText == "") {
                hasLong = true;
                break;
            }
        }
    }

    return hasLong;
}

window.isDocumentDigest = function() {
    var contentText = getText();
    var imgCount = getImgCount();
    return hasIframe() ||
    checkText(contentText) ||
    (imgCount > 1) ||
    (contentText.length > 2 && imgCount > 0) ||
    hasLongImg();
}

})();
;