(function(window){
var regexps = {
    unlikelyCandidates : /combx|comment|community|disqus|extra|foot|header|menu|remark|rss|shoutbox|sidebar|sponsor|ad-break|agegate|pagination|pager|popup|tweet|twitter/i,
    okMaybeItsACandidate : /and|article|body|column|main|shadow/i,
    positive : /article|body|content|entry|hentry|main|page|pagination|post|text|blog|story/i,
    negative : /combx|comment|com-|contact|foot|footer|footnote|masthead|media|meta|outbrain|promo|related|scroll|shoutbox|sidebar|sponsor|shopping|tags|tool|widget/i,
    extraneous : /print|archive|comment|discuss|e[\-]?mail|share|reply|all|login|sign|single/i,
    divToPElements : /<(a|blockquote|dl|div|img|ol|p|pre|table|ul)/i,
    replaceBrs : /(<br[^>]*>[ \n\r\t]*){2,}/gi,
    replaceFonts : /<(\/?)font[^>]*>/gi,
    trim : /^\s+|\s+$/g,
    normalize : /\s{2,}/g,
    killBreaks : /(<br\s*\/?>(\s|&nbsp;?)*){1,}/g,
    videos : /http:\/\/(www\.)?(youtube|vimeo)\.com/i,
    skipFootnoteLink : /^\s*(\[?[a-z0-9]{1,2}\]?|^|edit|citation needed)\s*$/i,
    nextLink : /(next|weiter|continue|>([^\|]|$)|»([^\|]|$))/i, // Match: next, continue, >, >>, » but not >|, »| as those usually mean last.
    prevLink : /(prev|earl|old|new|<|«)/i,
    candidateHTags : /<(h1|h2|h3|h4|h5|h6)/i,
    // CHINA DAILY（中国日报）的标题是在td标签里
    candidateOtherTitleTags : /<(div|p|td|span|header)/i,
    // 微信推荐的腾讯新闻页面标题在div标签里
    candidateOtherTitleTagsForMicromsg : /<(div|p)/i,
    // 太平洋汽车网 的标题是p标签，<p class="m-hd-tit">，所以增加tit关键字
    maybeTitle : /title|tit|head/i,
    // com.tencent.mobileqq中，标题的父级标签className包含media关键字，所以这里去除media
    unlikelyTitle : /combx|comment|contact|foot|footer|footnote|meta|promo|related|scroll|shoutbox|sponsor|utility|tags|widget|sub|relate|irelate|relative|relevance|relevant|relation|recommend|author|banner/i,//参照FindContent.js中negativeRe的定义
    unlikelyTitleTag : /footer/i,
    blockTags : /<(p|div|ul|ol|li|dl|dt|dd|h1|h2|h3|h4|h5|h6)/i,
    inlineTags : /<(a|span|em|strong|img|var)/i
};

function getDigestContent() {
    LbDebug("[getDigestContent] start, packageName : " + packageName);

    var digestContent = new Object();
    var url = window.location.href;

    // 先通过Pattern寻找
    var patternResult = null;
    var pattern = getPattern(packageName);
    if (pattern) {
        patternResult = LbProcessPattern(pattern);
    }

    // 替换懒加载的src属性值
    reloadImageElement();

    // 1. 获取channel（抓取订阅号的名称，有用，非必选，仅通过pattern抓取）
    if (patternResult && patternResult.channel) {
        digestContent.channel = patternResult.channel;
    }

    // 2. 获取imgUrl（首图）
    if (patternResult && patternResult.image) {
        digestContent.imgUrl = patternResult.image;
    }

    // 3. digestSummay（正文）
    getDigestSummay(digestContent, patternResult);

    // 给url统一添加"http:"头部，不以"http:"开头的话LbShareAdapter无法识别。
    if (digestContent.imgUrl) {
        var a = document.createElement("a");
        a.href = digestContent.imgUrl;
        digestContent.imgUrl = a.href;
    }

    // 4. 获取title（标题）
    getDigestTitle(digestContent, patternResult);

    // 5. 获取url
    getDigestUrl(digestContent, patternResult, url);
    if (!digestContent.digestSummary || digestContent.digestSummary.length == 0) {
        var bodyContent = document.body.innerText.trim().replace(/[\r\n]/g, "");
        var titleIndex = bodyContent.indexOf(digestContent.title);
        var currIndex = 0;
        if (titleIndex != -1) {
            currIndex = titleIndex + digestContent.title.length;
        }
        var summary = bodyContent.substring(currIndex);
        digestContent.digestSummary = summary;
    }
    digestContent.title = encodeURIComponent(digestContent.title);
    digestContent.isDigest = isDocumentDigest() ? 1 : 0;
    return digestContent;
}

function getDigestUrl(digestContent, patternResult, url) {
    // 懂球帝中打开的是第三方网址，里面的https://template.dongqiudi.com/news.html，URL设置为空，以mht文件内容为准。
    if ((patternResult && patternResult.ignoreUrl) || (packageName == "com.dongqiudi.news")) {
        digestContent.url = "";
    } else {
        // 百度新闻
        url = trimUrlByMark(url, "news.baidu.com", "?_k=");

        // 微信公众平台
        url = trimUrlByMark(url, "mp.weixin.qq.com", "&key=");
        digestContent.url = url;
    }
    LbDebug("url == : " + url);
}

function getDigestTitle(digestContent, patternResult) {
    try {
        if (patternResult && patternResult.title) {
            digestContent.title = patternResult.title;
        } else {
            digestContent.title = getTitle();
        }
        digestContent.title = digestContent.title.trim();
    } catch (e) {
        LbDebug("[getDigestContent] Get title error : " + e);
    }
}

function getDigestSummay(digestContent, patternResult) {
    // 如果是通过LbGrabContent抓取正文，并且第2步没有获取到imgUrl，则这里也要取一下imgUrl
    // 对于高度判断逻辑源于汽车之家的一篇长文触发取词后显示分词很慢，页面下方取词时间最长到16s,解决由于正文过多引起遍历内容拦截功能缓慢
    LbDebug("Summary offsetHeight:" + document.body.offsetHeight + ",width:" + window.screen.width);
    if (window.screen.width > 0 && document.body.offsetHeight / window.screen.width > 100) {
        if (document.body.innerText.length > 500) {
            digestContent.digestSummary = document.body.innerText.substring(0, 500);
        } else {
            digestContent.digestSummary = document.body.innerText;
        }
    } else if (patternResult && patternResult.body) {
        digestContent.digestSummary = patternResult.body;
    } else if (packageName != "com.tencent.mm") {
        try {
            var contentObj = LbGrabContent(LbCloneDocument(packageName));
            if ((!digestContent.imgUrl || digestContent.imgUrl.trim().length == 0)
                && (document.domain != "feeds.cloud.huawei.com")) {
                // 如果是UC浏览器的收藏，需要对首图的url进行预处理才能正常显示
                digestContent.imgUrl = trimUrlByMark(contentObj.contentImg, "image.uczzd.cn", "?id=");
            }
            digestContent.digestSummary = contentObj.contentText;
        }  catch(e){
             LbDebug("[getDigestContent] LbGrabContent error : " + e);
        }
    }
}

function trimUrlByMark(originalUrl, domain, mark){
    LbDebug("[trimUrlByMark] originalUrl: " + originalUrl)
    if( originalUrl != null && originalUrl.indexOf(domain) >= 0 && originalUrl.indexOf(mark) >=0){
        var trimUrl = originalUrl.substring(0,originalUrl.indexOf(mark));
        LbDebug("[trimUrlByMark] after trim: " + trimUrl)
        return trimUrl;
    }else{
        return originalUrl;
    }
}

function reloadImageElement() {
    var items = document.getElementsByTagName("img");
    for(var i = 0; i < items.length; i++){
        var imgDataSrc = items[i].getAttribute('data-src');
        if (imgDataSrc) {
         items[i].setAttribute('src', imgDataSrc);
       };
    }
}

function getTitle() {
    // 从body中查找title
    var titleCandidates = getTitleFromBody();

    // 再按照字体大小补充
    if (titleCandidates.length > 0) {
        supplyTitleCandidatesWithFontSize(titleCandidates);
        LbDebug("supply titleCandidates: " + titleCandidates.join());
    }

    // 从head中查找title
    var headTitle = getTitleFromHeadTitle();
    // 长度>5（经验值，如新浪财经的head.title就是“财经新闻”，这种就不参与相似度计算了）
    var headTitleValid = isTitleValid(headTitle) && getWordCount(headTitle) > 5;

    // 从head.meta中查找title
    var metaTitle = getTitleFromHeadMeta();
    // 长度>5（经验值）
    var metaTitleValid = isTitleValid(metaTitle) && getWordCount(metaTitle) > 5;

    var topTitleCandidate = null;
    // 豆瓣APP文章title在H1标签内。
    // 因新浪新闻title有重复的className，正确的在H1标签内。
    if (packageName == "com.douban.frodo" || packageName == "com.sina.news") {
         return titleCandidates[1].innerText;
    }
    // ZAKER 存在tpl_title不为空则以tpl_title为准
    if (packageName == "com.myzaker.ZAKER_Phone" &&
        document.getElementById("tpl_title") != null) {
        return document.getElementById("tpl_title").textContent;
    }
    for (var i = 0; i < titleCandidates.length; i++) {
        var similarity = getSimilarity(headTitleValid, metaTitleValid, titleCandidates[i], headTitle, metaTitle);

        // 如果100%相似，即相同，则直接返回该标题。
        if (similarity == 1) {
            return titleCandidates[i].innerText;
        }

        // 否则，只有当相似度>=0.4时才加分。
        else if (similarity >= 0.4) {
            titleCandidates[i].titleObject.score += 10 * similarity;
        }

        // 标题一般情况下都靠近前面，所以越靠后的标题，积分递减
        titleCandidates[i].titleObject.score -= i;

        if (!topTitleCandidate || titleCandidates[i].titleObject.score > topTitleCandidate.titleObject.score) {
            topTitleCandidate = titleCandidates[i];
        }

        // 如果候选标题都超过10个以上，10以后的候选标题就不用再去遍历了。
        if (i > 10) {
            break;
        }
    }
    if(topTitleCandidate){
        return topTitleCandidate.innerText;
    } else if(headTitleValid){
        // 由于知乎在webview中获取的title标签中的标题都是不对的，这里直接过滤掉
        if (packageName == "com.zhihu.android"){
            return "";
        }
        return headTitle;
    } else if(metaTitleValid){
        return metaTitle;
    } else{
        return getTitleFromTopTenBlock();
    }
}

function getSimilarity(headTitleValid, metaTitleValid, titleCandidate, headTitle, metaTitle) {
    var similarity = 0;
    // 如果head.title有效
    if (headTitleValid) {
        similarity = calcSimilarity(titleCandidate.innerText, headTitle);
    }

    // 如果存在metaTitle，则计算metaTitle与候选标题的相似度
    if (metaTitleValid) {
        var tmp = calcSimilarity(titleCandidate.innerText, metaTitle);

        // 如果相似度比上面head.title的相似度更高，则使用此相似度。
        if (tmp > similarity) {
            similarity = tmp;
        }
    }
    LbDebug("similarity: " + similarity);
    return similarity;
}

/**
 *从head中获取标准的标题内容
 */
function getTitleFromHeadTitle() {
    return document.title;
};

function getTitleFromHeadMeta() {
    // 华尔街见闻，部分文章的head.meta中标题描述，这里增加逻辑：如果meta包含name属性，并且name属性值包含title关键字，则取该meta的content作为Title。
    // 手机新浪网head.title与文章的标题相似度比较低，head.meta中也有标题，增加head.meta与文章候选标题的相似度比较。
    var metas = document.getElementsByTagName("meta");

    // 先看有没有property属性包含title的meta【手机新浪网：<meta property="og:title" content="中国真的对印妥协后撤百米？">】
    for (var i = 0; i < metas.length; i++) {
        if (metas[i].getAttribute("property") && metas[i].getAttribute("property").search(/title/i) != -1) {
            var title = metas[i].getAttribute("content");
            if (isTitleValid(title)) {
                return title;
            }
        }
    }

    // 再看有没有name属性包含title的meta【华尔街见闻：<meta name="wscn-share-title" content="每日机构观点汇总 | 美国CPI或使欧/美进一步回调 逢低买入时机到了！？">】
    for (var i = 0; i < metas.length; i++) {
        if (metas[i].getAttribute("name") && metas[i].getAttribute("name").search(/title/i) != -1) {
            var title = metas[i].getAttribute("content");
            if (isTitleValid(title)) {
                return title;
            }
        }
    }
    return null;
}

function getTitleFromBody() {
    var titleCandidates = [];
    var nodes = document.getElementsByTagName('*');
    // 百度浏览器从<div id="ssr-content-wrapper" 取title
    if (packageName == "com.baidu.browser.apps"
        && document.getElementById('ssr-content-wrapper') != null) {
        nodes = document.getElementById('ssr-content-wrapper');
    }
    for (var i = 0; i < nodes.length; ++i) {
        var node = nodes[i];
        var tagName = "<" + node.tagName;

        // 如果是h1-h6标签
        if (tagName.search(regexps.candidateHTags) !== -1) {
            // 如果标签自身有效，并且父级标签没有不像标题的，则加入候选
            if (isTitleTagValid(node) && !unlikeTitleParent(node)) {
                pushToCandidates(node, titleCandidates);
            }
        }
        else if (tagName.search(regexps.candidateOtherTitleTags) !== -1) {
            // 如果是div、p、span等标签
            if (isTitleTagValid(node) && (likeTitleSelf(node) || likeTitleSelf(node.parentNode)) && !unlikeTitleParent(node)) {
                // 如果标签内包含有块级子标签，则该标签不应该是标题，忽略
                if (node.innerHTML.search(regexps.blockTags) === -1) {
                    pushToCandidates(node, titleCandidates);
                } else {
                    LbDebug(node.tagName + " with className(" + node.className + "), id(" + node.id + ") contains block tags, ignore.");
                }
            }
        }
    }

    return titleCandidates;
}

function supplyTitleCandidatesWithFontSize(titleCandidates) {
    LbDebug("titleCandidates: " + titleCandidates.join());

    // 获取已有的候选title标签的最大字体大小
    var maxFontSize = 0;
    for (var i = 0; i < titleCandidates.length; i++) {
        var tempFontSize = parseInt(getComputedStyle(titleCandidates[i], undefined).getPropertyValue("font-size"));
        if (tempFontSize > maxFontSize) {
            maxFontSize = tempFontSize;
        }
    }
    LbDebug("maxFontSize: " + maxFontSize);

    // 遍历10次有效的div、p和h标签，如果标签字体大小大于等于已有候选的最大字体，插入到原有的候选数组中
    var count = 0;
    var otherNodes = document.getElementsByTagName('*');
    for (var i = 0; i < otherNodes.length; i++) {
        if (count > 10) {
            break;
        }
        var node = otherNodes[i];
        var tagName = "<" + node.tagName;

        // 如果是div、p、H标签
        if (tagName.search(regexps.candidateOtherTitleTagsForMicromsg) != -1 || tagName.search(regexps.candidateHTags) !== -1) {
            // 不存在block标签
            if (node.innerHTML.search(regexps.blockTags) == -1 && isTitleTagValid(node)) {
                count++;
                var fontSize = parseInt(getComputedStyle(node, undefined).getPropertyValue("font-size"));
                if (fontSize >= maxFontSize) {
                    var position = titleCandidates.length;
                    for (var j = 0; j < titleCandidates.length; j++) {
                        var candidate = titleCandidates[j];
                        var compare = candidate.compareDocumentPosition(node);

                        /*
                         node在某个候选之前，执行插入数组
                         NodeA.compareDocumentPosition(NodeB)
                         Bits         Number           Meaning
                         000000         0              元素一致
                         000001         1              节点在不同的文档（或者一个在文档之外）
                         000010         2              节点 B 在节点 A 之前
                         000100         4              节点 A 在节点 B 之前
                         001000         8              节点 B 包含节点 A
                         010000         16             节点 A 包含节点 B
                         100000         32             浏览器的私有使用
                        */
                        if (compare == 2 || compare == 10) {
                            position = j;
                            break;
                        }
                    }
                    if(!node.titleObject) {
                        node.titleObject = {
                            score : 0
                        };
                    }
                    LbDebug("insert " + tagName + " node.className: " + node.className + " " + node.id + ", fontSize: " + fontSize + " at position: " + position);
                    titleCandidates.splice(position, 0, node);
                }
            }
        }
    }
    return titleCandidates;
}

/**
 * 微信推荐的腾讯新闻页面，标题是div标签，且className是一个随机数，这边选取前10个div或者P标签的内容，选择字体最大的
 */
function getTitleFromTopTenBlock() {
    var tempCandidates = [];
    var otherNodes = document.getElementsByTagName('*');
    for (var i = 0; i < otherNodes.length; ++i) {
        if (tempCandidates.length >= 10) {
            break;
        }
        var node = otherNodes[i];
        var tagName = "<" + node.tagName;

        // 如果是div、p标签
        if (tagName.search(regexps.candidateOtherTitleTagsForMicromsg) !== -1) {
            if (node.innerHTML.search(regexps.blockTags) === -1 && isTitleTagValid(node)) {
                pushToCandidates(node, tempCandidates);
            }
        }
    }

    // 取出字体最大的一个快标签
    if (tempCandidates.length > 0) {
        var tempNode = tempCandidates[0];
        for (var i = 1; i < tempCandidates.length; ++i) {
            if (parseInt(getComputedStyle(tempCandidates[i],undefined).getPropertyValue("font-size")) >
                    parseInt(getComputedStyle(tempNode,undefined).getPropertyValue("font-size"))) {
                tempNode = tempCandidates[i];
            }
        }
        return tempNode.innerText;
    }
    return "";
}

/**
 * 判断标签自身是否像title标签
 */
function likeTitleSelf(tag) {
    var matchString = tag.className + tag.id;
    if(matchString.search(regexps.maybeTitle) != -1 && matchString.search(regexps.unlikelyTitle) == -1){
        return true;
    }
    return false;
}

/**
 * 判断该标签的父级（一直向上找）是不是不像title标签
 */
function unlikeTitleParent(tag) {
    var srcTag = tag;
    tag = tag.parentNode;
    var matchString;
    while(tag){
        if(tag == document.documentElement || tag == document.body){
            break;
        }
        matchString = tag.className + tag.id;
        if (typeof(matchString) == 'string' && (matchString.search(regexps.unlikelyTitle) != -1 || tag.tagName.search(regexps.unlikelyTitleTag) != -1)) {
            LbDebug("Parent tag " + tag.tagName + " with "+ tag.className + " is not like a title tag.");
            return true;
        }

        // 如果标题被a标签包裹，并且a标签的文本与标题是包含但不等于的关系，则减分。
        if (tag.tagName == 'A') {
            var textOfA = tag.innerText;
            var textOfSrcTag = srcTag.innerText;
            if (isTitleValid(textOfA) && isTitleValid(textOfSrcTag) && textOfA.search(textOfSrcTag) != -1 && textOfA != textOfSrcTag) {
                srcTag.titleObject = {
                    score : -5
                };
            }
        }
        tag = tag.parentNode;
    }
    return false;
}

/**
 * 判断该标签是否满足title标签的要求
 * 1.不包含链接；（条件不合理，已删除）2.标签可见；3.标签内容innerText有效。
 */
function isTitleTagValid(titleTag) {
    // 今日头条，过滤青云计划的标题。
    if (packageName == "com.ss.android.article.news" && titleTag.getElementsByClassName('green-name').length > 0) {
       return false;
    }
    // 如果标签不可见，视为无效
    if (!isTagVisible(titleTag)) {
        return false;
    }

    // 标签内容innerText有效
    if (!isTitleValid(titleTag.innerText)) {
        return false;
    }
    return true;
}

function isTitleValid(title) {
    return title != null && title.replace(regexps.trim, "").length > 0 && title != undefined;
}

function pushToCandidates(tag, titleCandidates){
    if(titleCandidates.indexOf(tag) != -1){
        return ;
    }
    if(!tag.titleObject) {
        tag.titleObject = {
            score : 0
        };
    }

    titleCandidates.push(tag);
}

/**
 * 计算两个字符串的相似度
 */
function calcSimilarity(str1, str2) {
    var map = {};
    var str1Len = str1.length;
    for(var i = 0 ; i < str1Len; i++) {
        if(map[str1[i]]){
            map[str1[i]][0]++;
        }else{
            var array = [1,0];
            map[str1[i]] = array;
        }
    }

    var str2Len = str2.length;
    for(var i = 0 ; i < str2Len; i++) {
        if(map[str2[i]]){
            map[str2[i]][1]++;
        }else{
            var array = [0,1];
            map[str2[i]] = array;
        }
    }

    var result = pointMulti(map)/sqrtMulti(map);
    LbDebug("[calcSimilarity] result : "+result);
    return result;
}

/**
 * 点乘法
 */
function pointMulti(map){
    var result = 0;
    for(var prop in map){
        result += map[prop][0] * map[prop][1];
    }
    return result;
}
function sqrtMulti(map){
    var result1 = 0;
    var result2 = 0;
    for(var prop in map){
        result1 += map[prop][0] * map[prop][0];
        result2 += map[prop][1] * map[prop][1];
    }
    var result = result1 * result2;
    result = Math.sqrt(result);
    return result;
}

window.getDigestContent = getDigestContent;

function getPattern(packageName) {
    if (browserList.indexOf(packageName) >= 0) {
        var domain = document.domain;
        if (DomainPattern[domain]) {
            return DomainPattern[domain];
        }
        for (var key in DomainPattern) {
            if (domain.indexOf(key) >= 0) {
                return DomainPattern[key];
            }
        }
        return null;
    } else {
        return PackagePattern[packageName];
    }
}

})(window)
;
