;
(function() {
    var contentImagePath = "";
    var list = null;

    function getOriginImage(img) {
        if (list == null) {
            list = document.getElementsByTagName("img");
        }

        for (var i = 0; i < list.length; i++) {
            if (img.src == list[i].src) {
                return list[i]
            }
        }
        return img;
    }

    function visiable(elem) {
        return !!( elem.offsetWidth || elem.offsetHeight || elem.getClientRects().length );
    };

    function checkImageSize(img) {
        if (!visiable(img)) {
            return false;
        }

        var w = img.width;
        if (w <= 1) {
            w = img.style.width.replace("px", "");
        }
        var h = img.height;
        if (h <= 1) {
            h = img.style.height.replace("px", ""); ;
        }

        if (w < 200 && h < 200) {
            return false;
        }

        if ((w / h > 2.9 || h / w > 2.9) && Math.min(w, h) < 200) {
            return false;
        }
        return true;
    }

    var pattern = {
        "com.ifeng.news2" : function(doc, content) {
            var divs = doc.getElementsByTagName("*");
            for (var i = 0; i < divs.length; i++) {
                if (divs[i].className.indexOf("slide") >= 0) {
                    if (divs[i].getAttribute("slidesrc")) {
                        return divs[i].getAttribute("slidesrc");
                    }
                }
                if (divs[i].className.indexOf("placeholder") >= 0) {
                    if (divs[i].getAttribute("src")) {
                        return divs[i].getAttribute("src");
                    }
                 }
            }
            return "";
        }
    };

    function getImageCommon(content) {
        var imgTags = content.getElementsByTagName('img');

        for (var i = 0; i < imgTags.length; i++) {
            var img = imgTags[i];
            if (!checkImageSize(getOriginImage(img))) {
                continue;
            }
            if (img.getAttribute('data-src')) {
                return img.getAttribute('data-src');
            }
            if (img.src) {
                if (img.src.indexOf("logo") > 0) {
                    continue;
                }
                if (img.src.indexOf("file:///android_asset") == 0) {
                    continue;
                }
                var elem = getOriginImage(img);

                // 解决华为浏览器头条号文章第一屏没图片，第二屏有图片懒加载时，稍后阅读右边图片一片黑问题
                if (packageName == "com.huawei.browser" && elem.naturalWidth && elem.naturalHeight) {
                    // 当前获取的图片为1*1像素的占位图时，获取img标签的data-url属性的值作为稍后阅读的图片
                    if (elem.naturalWidth == 1 && elem.naturalHeight == 1 && elem.getAttribute('data-url')) {
                        return elem.getAttribute('data-url');
                    }
                }
                return img.src;
            }
            return img.src;
        }
    }

    function getImage(doc, content, packageName) {
        var path = "";
        if (typeof pattern[packageName] == "function") {
            path = pattern[packageName](doc, content);
        }
        if (path) {
            return path;
        }
        path = getImageCommon(content);
        return path;
    }

//==============================================================================

    var regexps = {
        illegalTag: /script|style/i,
        unlikelyCandidatesRe: /combx|modal|lightbox|comment|disqus|foot|header|menu|meta|nav|shoutbox|sidebar|sponsor|social|teaserlist|time|tweet|twitter|recommend/i,
        okMaybeItsACandidateRe: /and|article|body|column|main/i,
        positiveRe: /article|body|content|entry|hentry|page|pagination|post|text/i,
        negativeRe: /combx|comment|contact|foot|footer|footnote|link|meta|promo|related|shoutbox|sponsor|utility|tags|widget|ads|reply|share/i,
        divToPElementsRe: /<(a|blockquote|dl|div|ol|p|pre|table|ul)/i,
        trimRe: /^\s+|\s+$/g,
        normalizeRe: /\s{2,}/g, // 找出目标字符串中所有空白字符
        killBreaksRe: /(<br\s*\/?>(\s|&nbsp;?)*){1,}/g, // <br\s*/?> /前允许出现0个或多个空白字符
        videoRe: /http:\/\/(www\.)?(youtube|vimeo|youku|tudou|56|yinyuetai)\.com/i
    };

    function foreach(list, f) {
        for (var i = 0; i < list.length; i++) {
            f(list[i])
        }
    }

    function prepDocument(document) {
        var frameTags = document.getElementsByTagName('frame');
        if (frameTags.length > 0) {
            var bestFrame = null;
            var bestFrameSize = 0;

            forEach(Array.prototype.slice.call(frameTags, 0),
            function(frame) {
                var frSize = frame.offsetWidth + frame.offsetHeight;
                var canAccessFrame = false;
                try {
                    frame.contentWindow.document.body;
                    canAccessFrame = true;
                } catch(e) {}

                if (canAccessFrame && frSize > bestFrameSize) {
                    bestFrame = frame;
                    bestFrameSize = frSize;
                }
            });

            if (bestFrame) {
                var myBody = document.createElement('body');
                myBody.innerHTML = bestFrame.contentWindow.document.body.innerHTML;
                myBody.style.overflow = 'scroll';
                document.body = myBody;

                var frameSet = document.getElementsByTagName('frameset')[0];
                if (frameSet) {
                    frameSet.parentNode.removeChild(frameSet);
                }
            }
        }

        // 将<br>替换成</p><p>
        document.body.innerHTML = document.body.innerHTML.replace(regexps.killBreaksRe,'</p><p>');
    };

    function grabArticle(document, preserveUnlikelyCandidates) {
        // 考虑下这里是否应该反向遍历
        var nodes = document.getElementsByTagName('*');
        var toRemove = getToRemove(nodes, preserveUnlikelyCandidates);
        for (var i = 0; i < toRemove.length; i++) {
            var eachNode = toRemove[i];
            if (eachNode.parentNode) {
                eachNode.parentNode.removeChild(eachNode);
            }
        }

        /*
         遍历所有段落，基于y坐标深度给它们分配分数，然后把它们的分数添加到父节点
         分数取决于逗号的数量，标签类名等等，最终也可能取决于链接密度
        */
        var allParagraphTags = document.getElementsByTagName("p");

        var candidates = getCandidates(allParagraphTags);
        // 计算完所有节点的分数后，遍历所有候选节点，找到分值最高的那个节点
        var bestCandidate = null;
        foreach(candidates,
        function(candidate) {
            // 基于链接密度放大最终的候选分数. 合格的正文内容应该有相对较小的链接密度 (5% or less) ，而且受该操作影响最小
            candidate.readability.contentScore = candidate.readability.contentScore * (1 - getLinkDensity(candidate));

            LbDebug('Candidate: ' + candidate + " (" + candidate.className + ":" + candidate.id + ") with score " + candidate.readability.contentScore);

            if (!bestCandidate || candidate.readability.contentScore > bestCandidate.readability.contentScore) bestCandidate = candidate;
        });

        // 如果还是没找到最优候选节点，就用body节点，要先拷贝body节点，然后才能修改它的内容
        if (bestCandidate === null || bestCandidate.tagName === 'BODY') {
            // 没有最优候选节点且没有body标签时直接退出
            if (!document.body) {
                return new Error('No body tag');
            }
            bestCandidate = document.createElement('DIV');
            bestCandidate.innerHTML = document.body.innerHTML;
            document.body.innerHTML = '';
            document.body.appendChild(bestCandidate);
            initializeNode(bestCandidate);
        }

        contentImagePath = getImage(window.document, bestCandidate, packageName);

       // 获取最优节点后，遍历它的同级节点获取可能相关的内容，包括之前删除的前言、被广告分割的内容等
        var articleContent = document.createElement('DIV');
        articleContent.id = 'readability-content';
        var siblingScoreThreshold = Math.max(10, bestCandidate.readability.contentScore * 0.2);
        var siblingNodes = bestCandidate.parentNode.childNodes;
        AddArticleContent(siblingNodes, articleContent, bestCandidate, siblingScoreThreshold);
       // 此时获取到了所有我们需要的内容，但还需进行一些清理操作才能将结果给出去
        prepArticle(articleContent);
        return articleContent;
    };

    function getToRemove(nodes, preserveUnlikelyCandidates) {
        var toRemove = [];
        for (var i = 0; i < nodes.length; ++i) {
            var eachNode = nodes[i];

            // 移除不像正文的节点
            var continueFlag = false;
            if (!preserveUnlikelyCandidates) {
                var unlikelyTag = eachNode.tagName.toLowerCase();
                if (unlikelyTag.search(regexps.illegalTag) !== -1) {
                    LbDebug("Removing unlikely candidate - " + unlikelyTag);
                    toRemove.push(eachNode);
                    continueFlag = true;
                }

                var unlikelyMatchString = eachNode.className + eachNode.id;
                if (unlikelyMatchString.search(regexps.unlikelyCandidatesRe) !== -1 && unlikelyMatchString.search(regexps.okMaybeItsACandidateRe) == -1 && eachNode.tagName !== 'HTML' && eachNode.tagName !== "BODY") {
                    LbDebug("Removing unlikely candidate - " + unlikelyMatchString);
                    toRemove.push(eachNode);
                    continueFlag = true;
                }

                if (window.getComputedStyle(eachNode).display == "none") {
                    LbDebug("Removing undisplayed candidate, unlikelyTag: " + unlikelyTag + ", unlikelyMatchString: " + unlikelyMatchString);
                    toRemove.push(eachNode);
                    continueFlag = true;
                }
            }

            if (!continueFlag && (eachNode.tagName === 'DIV' || eachNode.tagName === 'SECTION')) {
                if (eachNode.innerHTML.search(regexps.divToPElementsRe) === -1) {
                    LbDebug("Altering div to p");
                    var pNewNode = document.createElement('p');
                    pNewNode.innerHTML = eachNode.innerHTML;
                    eachNode.parentNode.replaceChild(pNewNode, eachNode);
                } else {
                    foreach(eachNode.childNodes,
                    function(childNode) {
                        // 节点为TEXT_NODE，并且节点内容长度>0时，才用span标签包裹，否则内容为空时用span标签包裹没有意义。
                        if (childNode.nodeType == 3 && childNode.nodeValue && childNode.nodeValue.length > 0) {
                            // 用span标签代替p标签，需要做更充分的测试
                            LbDebug("replacing text node with a span tag with the same content.");
                            var spanTag = document.createElement('span');
                            spanTag.innerHTML = childNode.nodeValue;
                            childNode.parentNode.replaceChild(spanTag, childNode);
                        }
                    });
                }
            }
        }
        return toRemove;
    }

    function getCandidates(allParagraphTags) {
        var candidates = [];
        for (var i = 0; i < allParagraphTags.length; ++i) {
            var paragraph = allParagraphTags[i];
            var parentNode = paragraph.parentNode;
            var grandParentNode = parentNode.parentNode;
            var innerText = getInnerText(paragraph);

            // 初始化父节点的内容可读性分数
            if (typeof parentNode.readability == 'undefined') {
                initializeNode(parentNode);
                candidates.push(parentNode);
            }

            // 初始化祖父节点的内容可读性分数
            if (typeof grandParentNode.readability == 'undefined') {
                initializeNode(grandParentNode);
                candidates.push(grandParentNode);
            }
            LbDebug("candidates : " + candidates);
            // 段落内可以呈现文本，也可以呈现图片。如果段落内的文本少于25个字并且没有图片，则过滤掉。
            if (innerText.length < 25 && paragraph.getElementsByTagName("img").length == 0) continue;

            var contentScore = 0;

            ++contentScore;

            // 替换中文逗号为英文逗号，中文逗号对应的unicode编码为\uff0c
            contentScore += innerText.replace('\uff0c', ',').split(',').length;

            contentScore += Math.min(Math.floor(innerText.length / 100), 3);

            // 父节点分数需要加上子节点分数，祖父节点分数需要加上子节点分数的一半
            parentNode.readability.contentScore += contentScore;
            grandParentNode.readability.contentScore += contentScore / 2;
        }
        return candidates;
    }

    function AddArticleContent(siblingNodes, articleContent, bestCandidate, siblingScoreThreshold) {
        var il = siblingNodes.length;
        for (var i = 0; i < il; i++) {
            var sibling = siblingNodes[i];
            var append = false;
            if (sibling === bestCandidate) {
                append = true;
            }
            if (typeof sibling.readability != 'undefined' && sibling.readability.contentScore >= siblingScoreThreshold) {
                append = true;
            }

            if (sibling.nodeName == 'P') {
                var linkDensity = getLinkDensity(sibling);
                var nodeContent = getInnerText(sibling);
                var contentLength = nodeContent.length;

                if (contentLength > 80 && linkDensity < 0.25) {
                    append = true;
                } else if (contentLength < 80 && linkDensity === 0 && nodeContent.search(/\.( |$)/) !== -1) {
                    append = true;
                }
            }

            if (append) {
                LbDebug("Appending node: " + sibling);
                articleContent.appendChild(sibling);
                i--;
                il--;
            }
        }
    }

    function getInnerText(e, normalizeSpaces) {
        var contentText = "";
        normalizeSpaces = (typeof normalizeSpaces == 'undefined') ? true: normalizeSpaces;
        contentText = (e.textContent || "").trim();

        if (normalizeSpaces) return contentText.replace(regexps.normalizeRe, " ");
        else return contentText;
    }

    function getCharCount(element, s) {
        s = s || ",";
        return getInnerText(element).split(s).length;
    }

function getLinkDensity(e) {
    var textLength = getInnerText(e).length;
    if (textLength == 0) {
        return 0;
    }
    var aTags = e.getElementsByTagName("a");
    var linkLength = 0;
    for (var i = 0,
    il = aTags.length; i < il; i++) {
        var href = aTags[i].getAttribute('href');

        // 过滤掉以下2种情况：<h2><a href="#menu"></a></h2>及<h2><a></a></h2>
        if (!href || (href.length > 0 && href[0] === '#')) continue;
        linkLength += getInnerText(aTags[i]).length;
    }
    return linkLength / textLength;
}

function getClassWeight(e) {
    var classWeight = 0;
    var matchStr = e.className + e.id;
    if (typeof(matchStr) == 'string') {
        var isParentNeg = isParentNegative(e);

        // 自身包含negativeRe，或者父级标签包含negativeRe，都减分
        if (matchStr.search(regexps.negativeRe) !== -1 || isParentNeg) classWeight -= 25;

        // 自身满足positiveRe，并且父级标签不包含negativeRe，则加分
        if (matchStr.search(regexps.positiveRe) !== -1 && !isParentNeg) classWeight += 25;
    }
    return classWeight;
}

/**
 * 判断该标签的父级（一直向上找）是不是不像正文
 */
function isParentNegative(element) {
    element = element.parentNode;
    var matchString;
    while(element){
        if(element == document.documentElement || element == document.body){
            break;
        }
        matchString = element.className + element.id;
        if (typeof(matchString) == 'string' && matchString.search(regexps.negativeRe) !== -1) {
            LbDebug("Parent element " + element.tagName + " with className["+ element.className + "] id[" + element.id + "] is negative.");
            return true;
        }
        element = element.parentNode;
    }
    return false;
}

    /**
     * 清理元素e中包含的tag标签
     */
    function clean(e, tag) {
        var tagList = e.getElementsByTagName(tag);
        var isEmbed = (tag == 'object' || tag == 'embed');

        for (var y = tagList.length - 1; y >= 0; y--) {
            // 不移除包含有youtube 和 vimeo videos的标签，因为人们往往想看这些东西
            if (isEmbed) {
                if (tagList[y].innerHTML.search(regexps.videoRe) !== -1) {
                    continue;
                }
            } else if (tag == 'form') {
                // 如果form标签内部有positiveRe关键字，则认为有可能是正文内容，这里就不做移除。（适配wind.android）
                if (tagList[y].innerHTML.search(regexps.positiveRe) !== -1) {
                 continue;
                }
            }
            tagList[y].parentNode.removeChild(tagList[y]);
        }
    }

    function cleanConditionally(e, tag) {
        var tagList = e.getElementsByTagName(tag);
        var curTagsLength = tagList.length;

        // i--反向遍历，这样我们就可以在不影响遍历的情况下同时删除节点。
        for (var i = curTagsLength - 1; i >= 0; i--) {
            var weight = getClassWeight(tagList[i]);

            LbDebug("Cleaning Conditionally " + tagList[i] + " (" + tagList[i].className + ":" + tagList[i].id + ")" + ((typeof tagList[i].readability != 'undefined') ? (" with score " + tagList[i].readability.contentScore) : ''));

            if (weight < 0) {
                tagList[i].parentNode.removeChild(tagList[i]);
            } else if ((getCharCount(tagList[i], ',') + getCharCount(tagList[i], '\uff0c')) < 10) {// 计算中文逗号和英文逗号的总和
                var p = tagList[i].getElementsByTagName("p").length;
                var img = tagList[i].getElementsByTagName("img").length;
                var li = tagList[i].getElementsByTagName("li").length - 100;
                var input = tagList[i].getElementsByTagName("input").length;

                var embedCount = 0;
                var embeds = tagList[i].getElementsByTagName("embed");
                for (var ei = 0,
                il = embeds.length; ei < il; ei++) {
                    if (embeds[ei].src && embeds[ei].src.search(regexps.videoRe) == -1) {
                        embedCount++;
                    }
                }

                var linkDensity = getLinkDensity(tagList[i]);
                var contentLength = getInnerText(tagList[i]).length;
                var toRemove = false;
                if (li > p && tag != "ul" && tag != "ol") {
                    toRemove = true;
                } else if (input > Math.floor(p / 3)) {
                    toRemove = true;
                } else if (contentLength < 25 && (img == 0)) {// 删掉img > 2是为了解决暴走日报 的暴走天天看栏目的文摘无法收藏问题
                    toRemove = true;
                } else if (weight < 25 && linkDensity > .2) {
                    toRemove = true;
                } else if (weight >= 25 && linkDensity > .5) {
                    toRemove = true;
                } else if (embedCount == 1 && contentLength < 75) {
                    toRemove = true;
                } else if(embedCount > 1) {
                    toRemove = true;
                } else {
                    LbDebug("other condition");
                }

                if (toRemove) {
                    tagList[i].parentNode.removeChild(tagList[i]);
                }
            }
        }
    }

    /**
     * 修正image标签和a标签的url值，将图片和链接的相对url转为绝对url
     */
    function fixLinks(tag) {
        if (!tag.ownerDocument.originalURL) {
            return;
        }

        function fixLink(link) {
            var fixed = url.resolve(tag.ownerDocument.originalURL, link);
            return fixed;
        }

        var i;
        var imgTags = tag.getElementsByTagName('img');
        for (i = imgTags.length - 1; i >= 0; --i) {
            var src = imgTags[i].getAttribute('src');
            if (src) {
                imgTags[i].setAttribute('src', fixLink(src));
            }
        }

        // 修正a标签的href值
        var aTags = tag.getElementsByTagName('a');
        for (i = aTags.length - 1; i >= 0; --i) {
            var href = aTags[i].getAttribute('href');
            if (href) {
                aTags[i].setAttribute('href', fixLink(href));
            }
        }
    }

    function cleanHeaders(e) {
        // 获取h1-h6标签的className + id的权重（链接密度getLinkDensity（）=0），如果权重小于0，则移除该节点
        for (var hIndex = 1; hIndex < 7; hIndex++) {
            var headers = e.getElementsByTagName('h' + hIndex);
            for (var n = headers.length - 1; n >= 0; --n) {
                if (getClassWeight(headers[n]) < 0 || getLinkDensity(headers[n]) > 0.33) {
                    headers[n].parentNode.removeChild(headers[n]);
                }
            }
        }
    }

    /**
     * 获取h1-h6标签，如果该标签没有nextSibling（元素之后紧跟的节点）则移除该标签
     */
    function cleanSingleHeader(e) {
        for (var hIndex = 1; hIndex < 7; hIndex++) {
            var headers = e.getElementsByTagName('h' + hIndex);
            for (var n = headers.length - 1; n >= 0; --n) {
                if (headers[n].nextSibling === null) {
                    headers[n].parentNode.removeChild(headers[n]);
                }
            }
        }
    }

    function prepArticle(articleContent) {
        // 移除标签的style属性
        cleanStyles(articleContent);
        killBreaks(articleContent);

        // 移除form标签，但如果form标签内部有positiveRe关键字则不移除
        clean(articleContent, 'form');

        // 移除object标签，但如果object标签内部有videoRe关键字则不移除
        clean(articleContent, 'object');

        // 如果只有一个h1标签，则移除
        if (articleContent.getElementsByTagName('h1').length === 1) {
            clean(articleContent, 'h1');
        }

        // 如果只有一个h2标签，这个标签可能被用作标题而不是子标题，由于已经有一个标题了，所以移除该标签
        if (articleContent.getElementsByTagName('h2').length === 1) clean(articleContent, "h2");

        // 移除iframe标签
        clean(articleContent, "iframe");

        // 移除h1-h6权重不合格的标签
        cleanHeaders(articleContent);
        cleanConditionally(articleContent, "table");
        cleanConditionally(articleContent, "ul");
        cleanConditionally(articleContent, "div");

        // 移除没有img，embed，object标签且没有文本内容的段落
        var articleParagraphs = articleContent.getElementsByTagName('p');
        for (var i = articleParagraphs.length - 1; i >= 0; i--) {
            var imgTagCount = articleParagraphs[i].getElementsByTagName('img').length;
            var embedCount = articleParagraphs[i].getElementsByTagName('embed').length;
            var objectCount = articleParagraphs[i].getElementsByTagName('object').length;

            if (imgTagCount == 0 && embedCount == 0 && objectCount == 0 && getInnerText(articleParagraphs[i], false) == '') {
                articleParagraphs[i].parentNode.removeChild(articleParagraphs[i]);
            }
        }

        // 移除h1-h6中没有nextSibling的标签
        cleanSingleHeader(articleContent);

        try {
            articleContent.innerHTML = articleContent.innerHTML.replace(/<br[^>]*>\s*<p/gi, '<p');
        } catch(e) {
            LbDebug("Cleaning innerHTML of breaks failed. This is an IE strict-block-elements bug. Ignoring.");
        }

        fixLinks(articleContent);
    }

    /**
     * 初始化节点的内容可读性分值
     */
    function initializeNode(element) {
        element.readability = {
            contentScore: 0
        };

        switch (element.tagName) {
        case 'ARTICLE':
            element.readability.contentScore += 10;
            break;

        case 'SECTION':
            element.readability.contentScore += 8;
            break;

        case 'DIV':
            element.readability.contentScore += 5;
            break;
        case 'PRE':
        case 'TD':
        case 'BLOCKQUOTE':
            element.readability.contentScore += 3;
            break;
        case 'ADDRESS':case 'DL':case 'DD':case 'DT':
        case 'OL':case 'UL':case 'LI':case 'FORM':
            element.readability.contentScore -= 3;
            break;
        case 'H1':case 'H2': case 'H3':case 'H4':case 'H5':case 'H6':case 'TH':
            element.readability.contentScore -= 5;
            break;
        }

        // 节点包含itemscope属性时内容可读性分值+5
        if (element.getAttribute && element.getAttribute("itemscope")) {
            element.readability.contentScore += 5;
            if (element.attributes.itemtype && /blog|post|article/i.test(element.getAttribute('itemtype'))) {
                element.readability.contentScore += 30;
            }
        }

        // 节点的内容可读性分值需加上节点className + id的权重
        element.readability.contentScore += getClassWeight(element);
    }

    function cleanStyles(e) {
        if (!e) return;

        if (typeof e.removeAttribute == 'function' && e.className != 'readability-styled') e.removeAttribute('style');

        // 遍历移除子节点的style属性
        var curNode = e.firstChild;
        while (curNode) {
            if (curNode.nodeType == 1) {
                if (curNode.className != "readability-styled") {
                    curNode.removeAttribute("style");
                }
                cleanStyles(curNode);
            }
            curNode = curNode.nextSibling;
        }
    }

    function killBreaks(element) {
        if (element.innerHTML) {
            element.innerHTML = element.innerHTML.replace(regexps.killBreaksRe, '<br />');
        }
    }

    window.LbGrabContent = function(document) {
        var doc = document;
        var articleContent = "";
        if (doc) {
            prepDocument(doc);
            articleContent = grabArticle(doc, false);
        }
        return {
            "contentText" : (getInnerText(articleContent) || "").trim().replace(/[\r\n\t]/g,""),//过滤掉正文里的空格、TAB、换行等
            "contentImg" : contentImagePath
        }
    }

})(window);
;
