;
(function() {
    var MAX_LEN = 300;
    var PADDING = 20; // webview 子标签实测扩展高度
    var CLIENT_HEIGHT_OF_PADDING = 625; // webview 子标签实测扩展高度匹配的webview屏幕高度

    function cutText(text) {
        if (text.length > MAX_LEN) {
            text = text.substr(0, MAX_LEN);
        }
        return text;
    }

    /**
     * 获取某个标签所属的层叠上下文
     */
    function getLocOfAncestor(element){
        while (element = element.parentNode) {
            if (element == document.body) {
                return element;
            }
            var cs = window.getComputedStyle(element);
            var position = cs.position;
            var zIndex = cs['z-index'];
            if (zIndex && zIndex != 'auto') {
                if (position == 'absolute' || position == 'fixed' || position == 'relative') {
                    return element;
                }
            }
        }
    }

    /**
     * 函数描述：获取指定坐标[x,y]的内容
     * 实现功能：按照z-index查找。
     */
    function getContentInPos(x, y) {
        // 找出所有设置了有效的z-index的标签
        getZindexChildren();

        // 对z-index按照从大到小排列
        sortZindexChildren(document.body, "-- ");

        // 获取文本
        return getContentByZindex(document.body, x, y);
    }

    /**
     * 函数描述：遍历整个页面，寻找所有有效的z-index的元素
     * 实现功能：遍历所有节点，寻找所有有效的z-index的元素。将该元素添加到所在的层叠上下文的标签的zindexChildren属性中；
     * 另外，如果该元素自身形成独立的层叠上下文，则也要初始化自己的zindexChildren。
     */
    function getZindexChildren(){
        // 页面根元素天生具有层叠上下文，称之为“根层叠上下文”。所以，初始化zindexChildren。
        document.body.zindexChildren = [];

        var DOMs = document.all;//获取所有DOM
        var ele;
        var cs;
        var position;
        var zIndex;

        // 遍历所有元素
        for (var i = 0;i < DOMs.length;i++) {
            ele = DOMs[i];
            cs = window.getComputedStyle(ele);
            position = cs.position;
            zIndex = cs['z-index'];

            // 如果不可见，过滤；
            if (cs.visibility == 'hidden' || cs.display == 'none') {
                continue;
            }

            // 自身可见，但父标签（及以上）不可见，也过滤；
            if (!isTagVisible(ele)) {
                continue;
            }

            // 当某个元素的 z-index 未显式定义或者被指定为 auto 时，该元素不会产生新的局部层叠上下文。
            // 所以，z-index定义成整数，并且position!= static，则为有效的z-index。
            if (zIndex && zIndex != 'auto' && position != 'static') {
                // 首先，标签自身形成一个层叠上下文
                ele.zindexChildren = [];

                // 然后，找到该标签所在层叠上下文，将ele添加到该层叠上下文的zindexChildren中
                if (position == 'absolute' || position == 'relative') {
                    // 寻找层叠上下文
                    var ancestor = getLocOfAncestor(ele);
                    ancestor.zindexChildren.push(ele);
                }
                else if (position == 'fixed') {
                    // 固定定位总是相对于浏览器窗口来进行定位的，它抛弃了它的父元素。
                    // 新的层级
                    document.body.zindexChildren.push(ele);
                }
            }
        }
    }

    /**
     * 函数描述：所有含有zindexChildren属性的标签，将zindexChildren的元素按照z-index从大到小进行排序
     * 参数说明：cnt代表当前层级，只为输出调试日志
     */
    function sortZindexChildren(element, cnt){
        if (isTimeOut()) {
            return;
        }
        element.zindexChildren = element.zindexChildren.sort(sortByZIndex);
        for (var i = 0; i < element.zindexChildren.length;i++) {
            if (isTimeOut()) {
                return;
            }
            if(element.zindexChildren[i].zindexChildren){
                sortZindexChildren(element.zindexChildren[i], cnt + "-- ");
            }
        }
    }

    /**
     * 函数描述：根据z-index由大到小进行文本查找
     */
    function getContentByZindex(element, x, y){
        var zIndex = window.getComputedStyle(element)['z-index'];
        var isAutoSelf = zIndex && zIndex == 'auto';

        // 对于z-index为auto的element，判断是否有通过getTxtAtPoint处理过
        var hasProcessAutoContent = false;

        // 根据z-index从大到小进行遍历。但是要考虑element自身的z-index是auto还是整数的两种不同处理（原则是：当父元素有设置z-index时，那么他的子元素的层级一定会比他高。）：
        // 如果element自身的z-index是auto，则zindexChildren中z-index为负数的是显示在element后面的；（即从前往后的排列顺序是：正z-index元素、element自身、负z-index元素。）
        // 如果element是整数，则zindexChildren中z-index为负数的是显示在element前面的。（即从前往后的排列顺序是：正z-index元素、负z-index元素、element自身。）
        for(var i = 0; i < element.zindexChildren.length;i++){
            if (isTimeOut()) {
            return null;
            }

            // 对于z-index >=0 时
            if( window.getComputedStyle(element.zindexChildren[i])['z-index'] >= 0){
                var tmpContent;

                // 如果该子元素内部还有层叠，则递归调用getContentByZindex
                if(element.zindexChildren[i].zindexChildren){
                    tmpContent = getContentByZindex(element.zindexChildren[i], x, y);
                }
                else {
                    // 如果该子元素内部没有层叠，则调用getTxtAtPoint
                    tmpContent = getTxtAtPoint(element.zindexChildren[i], x, y);
                }

                // 如果tmpContent不为null，则返回（注意：tmpContent为""时也同样返回）。（下同）
                if(tmpContent != null || tmpContent != undefined) {
                    return tmpContent;
                }
            }else {
                // 当zindexChildren[i]的z-index为负数时，就要考虑element自身z-index是auto还是整数的区别了。
                // 如果element的z-index是整数，肯定是在最后，所以不用在这个for循环里处理，而是退出循环后再处理。
                // 如果element的z-index是auto，则显示在z-index为负数的子元素前面。处理如下：
                if(isAutoSelf){
                    // 如果z-index为auto的element已经处理过，则与上面z-index>=0时处理一致。
                    if(hasProcessAutoContent == true){
                        var tmpContent;
                        if(element.zindexChildren[i].zindexChildren) {
                            tmpContent = getContentByZindex(element.zindexChildren[i], x, y);
                        }else {
                            tmpContent = getTxtAtPoint(element.zindexChildren[i], x, y);
                        }
                        if(tmpContent != null || tmpContent != undefined) {
                            return tmpContent;
                        }
                    }

                    // 如果z-index为auto的element未处理过，则通过getTxtAtPoint获取内容
                    else {
                        hasProcessAutoContent = true;
                        var tmpContent = getTxtAtPoint(element, x, y);
                        if(tmpContent != null || tmpContent != undefined) {
                            return tmpContent;
                        }
                    }
                }
            }
        }

        if (!hasProcessAutoContent) {
            hasProcessAutoContent = true;
            var tmpContent = getTxtAtPoint(element, x, y);
            if(tmpContent != null || tmpContent != undefined) {
                return tmpContent;
            }
        }
    }

    /**
     * 函数描述：返回按压点[x,y]处的文本
     * 实现功能：将[x,y]转换成网页上的按压点坐标。
     */
    function getTextAtPoint(x, y) {
        // 传入的[x,y]是在WebView控件上的绝对坐标，即以WebView左上为[0,0]。
        // 所以，[x,y]的取值范围是[0,0]-[WebView宽度,WebView高度]。
        LbDebug("[getTextAtPoint] Original x,y : [" + x + ", " + y + "]");
        LbDebug("[getTextAtPoint] currentScale : " + currentScale + ", pixDensity : " + pixDensity);
        LbDebug("[getTextAtPoint] document.body.scrollLeft : " + document.body.scrollLeft + ", document.body.scrollTop : " + document.body.scrollTop);

        // 转换到网页上对应的坐标时，需要做如下换算：
        // 第一步，除以网页缩放比例；
        x /= currentScale;
        y /= currentScale;
        if(window.webviewScrollX != undefined && window.webviewScrollY != undefined) {
            x += window.webviewScrollX/currentScale;
            y += window.webviewScrollY/currentScale;
        }
        LbDebug("[getTextAtPoint] actual x,y : [" + x + ", " + y + "]");

        // 针对UC内核的浏览器，无法计算出webview真实的起始坐标，通过js获取按压文本是错误的，所以这里直接返回空，交OCR处理
        if (window.hwGetTextUseOcr) {
            return "";
        } else {
            /*
             util.js中getElCoordinate方法计算元素坐标时，不考虑scrollTop/scrollLeft了，所以这里也不需要考虑scrollTop/scrollLeft。
             第二步，如果网页有滚动，要加上滚动的距离。
             x += (document.body.scrollLeft || window.pageXOffset || document.documentElement.scrollLeft || 0);document.body.scrollLeft是网页被卷去的左
             y += (document.body.scrollTop || window.pageYOffset || document.documentElement.scrollTop || 0);document.body.scrollTop是网页被卷去的高
            */
            // 获取[x,y]坐标处的内容
            return getContentInPos(x, y) || "";
        }
    }

    /**
     * 函数描述：对标签按照z-index排序
     * 实现功能：分别获取a、b标签的z-index值，按降序排列。
     */
    function sortByZIndex(a,b){
        var aZIndex;
        if(a.nodeType != Node.ELEMENT_NODE) {
            aZIndex = 0;
        }else{
            aZIndex = window.getComputedStyle(a)['z-index'];
            aZIndex = (aZIndex == 'auto' ? 0 : aZIndex);
        }
        var bZIndex;
        if(b.nodeType != Node.ELEMENT_NODE) {
            bZIndex = 0;
        }else{
            bZIndex = window.getComputedStyle(b)['z-index'];
            bZIndex = (bZIndex == 'auto' ? 0 : bZIndex);
        }
        return bZIndex - aZIndex;
    }

    /**
     * 函数描述：从element标签往内逐步寻找按压点所处位置的文本。
     * 实现功能：如果按压在element的子标签上，则递归调用此方法，继续往子标签上寻找。
     * 备注：要区分返回的是""还是null
     */
    function getTxtAtPoint(element, x, y) {
        var elePos = window.getComputedStyle(element).position;
        var margin;
        if (window.isTencentBrowser) {
            margin = PADDING * document.documentElement.clientHeight / CLIENT_HEIGHT_OF_PADDING;
        } else {
            margin = 0;
        }

        if (!getElCoordinate(element)){
            return null;
        }

        // x,y是否在element的区域内
        var inRect;
        if (window.isTencentBrowser) {
            inRect = isInRect(element.coordinate.left, element.coordinate.top - margin , document.documentElement.clientWidth , element.coordinate.bottom + margin, x, y);
        } else {
            inRect = isInRect(element.coordinate.left, element.coordinate.top - margin, element.coordinate.right , element.coordinate.bottom + margin, x, y);
        }
        if (!inRect) {
             LbDebug("not inRect, " + getElementLog(element));
            var overflow = window.getComputedStyle(element).overflow;

            // 标签的overflow为hidden时，说明内容会被修剪，并且其余内容是不可见的。则不再对不在范围内的子元素进行遍历。
            if(overflow == 'hidden') {
                return null;
            }
        }

        // 将NodeList类型的子节点集合childNodes转换成Array
        var childs = Array.prototype.slice.call(element.childNodes);

        var childCount = childs.length;

        // 如果无子节点，可能是起空行的作用
        if (childCount == 0) {
            // 是按压区域，返回空内容。
            if (inRect) {
                return "";
            }
            else {
                // 不在按压区域，返回NULL。
                return null;
            }
        }
        else if (childCount == 1) {
            // 如果只有一个子节点，分两种情况：1.如果是TEXT类型，则返回文本；2.如果是ELEMENT类型，则继续向下找。
            // 如果这个子节点是标签类型，则到该标签内继续寻找
            if (childs[0].nodeType == Node.ELEMENT_NODE && window.getComputedStyle(childs[0]).visibility != 'hidden') {
                var tmpContent = getTxtAtPoint(childs[0], x, y);

                // 如果子标签返回的是有效内容（即获取到了内容），则返回内容，否则不返回。
                if (tmpContent != null && tmpContent != undefined) {
                    return tmpContent;
                }
            }

            // 如果是文本类型子节点则返回文本
            else if (childs[0].nodeType == Node.TEXT_NODE) {
                // 如果是文本节点，在按压区域内，则返回文本内容；
                if (inRect) {
                    return (childs[0].textContent || "").trim();
                }
                else {
                    // 文本节点不在按压区域，返回NULL。
                    return null;
                }
            }
        }
        else {
            // 如果存在多个子节点，看按压点在哪个节点上。
            for (var i = 0; i < childCount; i++) {
                if (isTimeOut()) {
                    return null;
                }

                // 判断[x,y]是否在节点i上
                if (childs[i].nodeType == Node.TEXT_NODE) {
                    // 如果按压点在该文本节点上，则寻找该文本节点前面与后面的元素，直至分别遇到块标签为止
                    if (isTouchOnText(childs[i], i, x, y)) {
                        return getTextFromNode(childs[i]);
                    }
                }
                else if (childs[i].nodeType == Node.ELEMENT_NODE && window.getComputedStyle(childs[i]).visibility != 'hidden') {
                    // 如果是块标签，则继续往子节点找
                    if (isBreakLine(childs[i])) {
                        var tmpContent = getTxtAtPoint(childs[i], x, y);

                        // 如果子标签返回的是有效内容（即获取到了内容），则返回内容，否则不返回。
                        if (tmpContent != null && tmpContent != undefined) {
                            return tmpContent;
                        }
                    }
                    else {
                        // 如果是行标签，则遍历该行标签的父标签内所有子节点，寻找该行标签前面与后面的元素，直到遇到块标签为止
                        getElCoordinate(childs[i]);

                        // 例外：如果行标签内包含img标签，对所有的img标签进行遍历，如果按压点在img标签上，则返回""，以进行后续的OCR识别。
                        var imgTags = childs[i].getElementsByTagName('img');
                        if (imgTags) {
                            for (var imgIdx = 0; imgIdx < imgTags.length; imgIdx++) {
                                if (getElCoordinate(imgTags[imgIdx])) {
                                    if (isInRect(imgTags[imgIdx].coordinate.left, imgTags[imgIdx].coordinate.top, imgTags[imgIdx].coordinate.right, imgTags[imgIdx].coordinate.bottom, x, y)) {
                                        return "";
                                    }
                                }
                            }
                        }
                        var right;

                        // 对于行标签，直接计算该行标签的区域，如果按压点在这个区域，则返回内容
                        if (window.isTencentBrowser) {
                            right = document.documentElement.clientWidth;
                        } else {
                            right = childs[i].coordinate.right;
                        }
                        if (isInRect(childs[i].coordinate.left, childs[i].coordinate.top - margin, right, childs[i].coordinate.bottom + margin, x, y)) {
                            return getTextFromNode(childs[i]);
                        }
                    }
                }
            }
        }

        // 当子节点遍历结束，还能走到这里，说明尚未返回内容。
        // 所以，如果按压点在element的区域内，则直接返回element的innerText；否则，返回NULL。
        if (inRect) {
            return element.innerText;
        } else {
            return null;
        }
    }

    /**
     * 函数描述：取前面的同级标签，并且是块标签
     * 实现功能：找该node之前的Element类型兄弟节点，并且是块标签类型。
     */
    function getBlockPreviousEleSibling(node) {
        var preSibling = node.previousElementSibling;
        while(preSibling){
            // 同级的块标签，有可能获取到的坐标是无效的。如有些情况下p标签的宽高都是0，这时该标签就无效，需要继续找其他兄弟标签。
            // 2018.01.09 如果preSibling标签有zindexChildren属性，说明是单独的层叠上下文，不能参与这里的计算。
            if(isBreakLine(preSibling) && getElCoordinate(preSibling) && preSibling.zindexChildren == undefined) {
                return preSibling;
            }
            preSibling = preSibling.previousElementSibling;
        }
    }

    /**
     * 函数描述：取node之后的同级标签，并且是块标签
     * 实现功能：找该node之后的Element类型兄弟节点，并且是块标签类型。
     */
    function getBlockNextEleSibling(node) {
        var nextSibling = node.nextElementSibling;
        while(nextSibling){
            // 同级的块标签，有可能获取到的坐标是无效的。如有些情况下p标签的宽高都是0，这时该标签就无效，需要继续找其他兄弟标签。
            // 2018.01.09 如果nextSibling标签有zindexChildren属性，说明是单独的层叠上下文，不能参与这里的计算。
            if(isBreakLine(nextSibling) && getElCoordinate(nextSibling) && nextSibling.zindexChildren == undefined) {
                return nextSibling;
            }
            nextSibling = nextSibling.nextElementSibling;
        }
    }

    /**
     * 函数描述：坐标是否落在某个文本上
     * 实现功能：寻找该文本的前一个块标签与后一个块标签，从而得到该文本的矩形区域范围。
     */
    function isTouchOnText(textNode, nodeIdxInParent, x, y) {
        // 如果该文本节点无文本，则直接返回false
        // 2017.08.23更新：有些文本节点无文本，但占据了空间，所以这里不能直接返回，需要计算出该空文本的文本节点区域。
        var left,top,right,bottom;

        // 前一个块标签
        var beforeBrkLineElement;

        // 后一个块标签
        var afterBrkLineElement;

        // 取前面的同级标签，并且是块标签
        beforeBrkLineElement = getBlockPreviousEleSibling(textNode);

        // 前一个块标签是父标签（或父父标签...）还是同级标签（自己的同级标签或者父标签的同级标签）
        var beforeBrkLineElementIsParent = false;

        // 前面无同级的块标签，则取是块标签的父级标签
        if(!beforeBrkLineElement) {
            beforeBrkLineElement = textNode.parentNode;
            beforeBrkLineElementIsParent = true;

            // 不是块标签；或者是块标签，但块标签的坐标无效。
            // 这时，先找父标签的前面同级标签，如果还不符合，再找父父标签，如果父父标签依然不符合，再找父父标签的前面同级标签，以此类推。
            // 2018.01.09 如果beforeBrkLineElement标签有zindexChildren属性，说明是单独的层叠上下文，不能参与这里的计算。
            while(!isBreakLine(beforeBrkLineElement) || !getElCoordinate(beforeBrkLineElement)) {
                // 如果父标签不符合要求，则寻找父标签的前面同级标签
                if (beforeBrkLineElementIsParent) {
                    var tmpBeforeEle = getBlockPreviousEleSibling(beforeBrkLineElement);

                    // 如果父标签前面有同级标签
                    if (tmpBeforeEle) {
                        beforeBrkLineElement = tmpBeforeEle;
                        beforeBrkLineElementIsParent = false;
                    }
                    else {
                        // 如果父标签前面没有同级标签，则找父标签的父标签
                        beforeBrkLineElement = beforeBrkLineElement.parentNode;
                        beforeBrkLineElementIsParent = true;
                    }
                }
                else {
                    beforeBrkLineElement = beforeBrkLineElement.parentNode;
                    beforeBrkLineElementIsParent = true;
                }
            }
        }

        if (beforeBrkLineElementIsParent) {
            // 如果是父级标签，则以父级标签的左上作为文本节点区域的左上坐标
            left = beforeBrkLineElement.coordinate.left;
            top = beforeBrkLineElement.coordinate.top;
        }
        else {
            // 有同级块标签，取该标签的左下作为文本节点区域的左上坐标
            // 上一个兄弟块标签的左下即是文本节点的左上
            left = beforeBrkLineElement.coordinate.left;
            top = beforeBrkLineElement.coordinate.bottom;
        }

        // 取后面的同级标签，并且是块标签
        afterBrkLineElement = getBlockNextEleSibling(textNode);

        // 后一个块标签是父标签（或父父标签...）还是同级标签（自己的同级标签或者父标签的同级标签）
        var afterBrkLineElementIsParent = false;

        // 后面无同级的块标签，则取父级的块标签
        if (!afterBrkLineElement) {
            afterBrkLineElement = textNode.parentNode;
            afterBrkLineElementIsParent = true;

            // 不是块标签；或者是块标签，但块标签的坐标无效。
            // 2018.01.09 如果afterBrkLineElement标签有zindexChildren属性，说明是单独的层叠上下文，不能参与这里的计算。
            while (!isBreakLine(afterBrkLineElement) || !getElCoordinate(afterBrkLineElement)) {
                if (afterBrkLineElementIsParent) {
                    var tmpAfterEle = getBlockNextEleSibling(afterBrkLineElement);
                    if (tmpAfterEle) {
                        afterBrkLineElement = tmpAfterEle;
                        afterBrkLineElementIsParent = false;
                    }
                    else {
                        afterBrkLineElement = afterBrkLineElement.parentNode;
                        afterBrkLineElementIsParent = true;
                    }
                } else {
                    afterBrkLineElement = afterBrkLineElement.parentNode;
                    afterBrkLineElementIsParent = true;
                }
            }
        }

        if (afterBrkLineElementIsParent) {
            // 如果是父级标签，则以父级标签的右下作为文本节点区域的右下坐标
            right = afterBrkLineElement.coordinate.right;
            bottom = afterBrkLineElement.coordinate.bottom;
        }
        else {
            // 如果是同级块标签，则以同级块标签的右上作为文本节点区域的右下坐标
            right = afterBrkLineElement.coordinate.right;
            bottom = afterBrkLineElement.coordinate.top;
        }
        return isInRect(left, top, right, bottom, x, y);
    }

    function getTextFromNode(node) {
        var siblingNodes = node.parentNode.childNodes;
        for(var i = 0; i< siblingNodes.length; i++) {
            if (isTimeOut()) {
                return "";
            }
            if(node == siblingNodes[i]) {
                return getTextFrom(node, i);
            }
        }
    }

    /**
     * 函数描述：寻找该行标签destElement前面与后面的元素，直到遇到块标签为止
     * 参数说明：nodeIdxInParent：node节点在父节点中的索引值
     * 实现功能：取该文本前后块标签中间的所有文本。
     */
    function getTextFrom(node, nodeIdxInParent) {
        var parentNode = node.parentNode;

        // 在指定节点之前的符合条件的节点
        var beforeNodes = [];

        // 在指定节点之后的符合条件的节点
        var afterNodes = [];

        // node节点的兄弟节点集合（包括自己）
        var siblingNodes = parentNode.childNodes;
        var siblingNodesCount = siblingNodes.length;

        // 在指定节点往前查找，如果是文本节点则push到beforeNodes；如果是行标签，则push到beforeNodes；遇到块标签，则退出while循环。
        var i = nodeIdxInParent -1;
        while (i>=0) {
            if (isTimeOut()) {
                break;
             }
            if (siblingNodes[i].nodeType == Node.TEXT_NODE) {
                beforeNodes.push(siblingNodes[i]);
            } else if (siblingNodes[i].nodeType == Node.ELEMENT_NODE) {
                if(isBreakLine(siblingNodes[i])){
                    break;
                }
                else {
                    beforeNodes.push(siblingNodes[i])
                }
            }
            i--;
        }

        // 把节点自身先push进来
        afterNodes.push(node);

        // 在指定节点往后查找，如果是文本节点则push到afterNodes；如果是行标签，则push到afterNodes；遇到块标签，则退出while循环。
        i = nodeIdxInParent + 1;
        while (i<siblingNodesCount) {
             if (isTimeOut()) {
                 break;
             }
            if (siblingNodes[i].nodeType == Node.TEXT_NODE) {
                afterNodes.push(siblingNodes[i]);
            } else if (siblingNodes[i].nodeType == Node.ELEMENT_NODE) {
                if(isBreakLine(siblingNodes[i])){
                    break;
                }
                else {
                    afterNodes.push(siblingNodes[i])
                }
            }
            i++;
        }

        // 取beforeNodes和afterNodes中的文本
        var text = "";
        for (var bi = beforeNodes.length -1; bi >=0; bi--) {
            if (isTimeOut()) {
                return "";
            }
            if (beforeNodes[bi].nodeType == Node.TEXT_NODE) {
                text += (beforeNodes[bi].textContent || "");
            } else if (beforeNodes[bi].nodeType == Node.ELEMENT_NODE) {
                text += (beforeNodes[bi].innerText || "");
            }
        }
        for (var ai = 0; ai < afterNodes.length; ai++) {
            if (isTimeOut()) {
                return "";
            }
            if (afterNodes[ai].nodeType == Node.TEXT_NODE) {
                text += (afterNodes[ai].textContent || "");
            } else if (afterNodes[ai].nodeType == Node.ELEMENT_NODE) {
                text += (afterNodes[ai].innerText || "");
            }
        }
        return text;
    }

    function getAllText() {
        var t = [];
        enumDocument(t, document.body);
        return t;
    }

    function getText(x, y) {
        var allText = getAllText();
        var text = getTextAtPoint(x, y);
        LbDebug("[getText] text.length : " + text.length);
        var index = allText.indexOf(text);

        var result = {};
        result.index = 0;
        result.dialogue = [];
        if (isTimeOut()) {
            return result;
        }

        if (index < 0) {
            result.dialogue.push({"txt": text.replace(/[\u200B][\r\n\t]/g,"").trim()});
            return result;
        }

        var startIndex = Math.max(index - 5, 0);
        var endIndex = Math.min(index + 5, allText.length - 1);
        var i = index - startIndex;

        result.index = i;
        for (var i = startIndex; i <= endIndex; i++) {
            result.dialogue.push({"txt": allText[i].replace(/[\u200B][\r\n\t]/g,"").trim()});
        }
        return result;
    }

    function isBreakLine(node) {
        if (node.nodeType != Node.ELEMENT_NODE) {
            return false;
        }

        var display = window.getComputedStyle(node).display;

        // br/li/td这些标签的display有可能是inline，所以这里直接排除掉。
        var tagName = ["BR", "LI", "TD"];
        if (tagName.indexOf(node.nodeName) >= 0) {
            return true;
        }
        if (display.indexOf("inline") >= 0) {
            for(var i =0; i < node.childNodes.length; i++) {
                if(isBreakLine(node.childNodes[i])) {
                    return true;
                }
            }
            return false;
        } else {
            return true;
        }
    }

    function allInlineChild(node) {
        var childs = node.getElementsByTagName("*");
        for (var i = 0; i < childs.length; i++) {
            var child = childs[i];
            if (isBreakLine(child)) {
                return false;
            }
        }
        return true;
    }

    function enumDocument(result, node) {
        if (node == null) {
            return;
        }
        if (node.nodeType == Node.TEXT_NODE) {
            var text = (node.textContent || "").trim();
            if (text) {
                result.push(cutText(text));
            }

            return;
        }
        if (node.nodeType != Node.ELEMENT_NODE) {
            return;
        }
        if (node.tagName && node.tagName.toLowerCase() == 'script') {
            return;
        }
        if (node.tagName && node.tagName.toLowerCase() == 'style') {
            return;
        }

        if (allInlineChild(node)) {
            var text = (node.innerText || "").trim();
            if (text) {
                result.push(cutText(text));
            }

            return;
        }

        var childs = node.childNodes;

        for (var i = 0; i < childs.length; i++) {
            enumDocument(result, childs[i]);
        }
    };

    window.grabWebviewText = getText;
})()
;