;
(function() {

    function cloneDocument() {
        var doc = document.implementation.createDocument(window.location.href, "html");
        cloneNode(doc.getElementsByTagName("html")[0], document.getElementsByTagName("html")[0]);
        return doc;
    }

    function cloneNode(destNode, srcNode) {
        cloneAttr(destNode, srcNode);
        var childs = srcNode.childNodes;
        for (var i = 0; i < childs.length; i++) {
            var srcEl = childs[i];
            if (srcEl.nodeType == document.ELEMENT_NODE) {

                var el = document.createElement(srcEl.nodeName);
                destNode.appendChild(el);
                cloneNode(el, srcEl);
            } else if (srcEl.nodeType == document.TEXT_NODE) {
                var el = document.createTextNode(srcEl.nodeValue);
                destNode.appendChild(el);
            }
        }
    }

    // 适配有效属性名的正则表达式
    var attrNameReg = /^[a-zA-Z_]+/;

    function cloneAttr(destNode, srcNode) {
        var attr = srcNode.attributes;
        for (var i = 0; i < attr.length; i++) {
            var name = attr[i].name.trim();
            var value = attr[i].value.trim();
            if (attrNameReg.test(name) && value != "") {
                destNode.setAttribute(name, value);
            }
        }
    }

    var ContentGrabBlackList = [
        "sogou.mobile.explorer.huawei",
        "sogou.mobile.explorer"
    ]

    window.LbCloneDocument = function(packageName) {
        if (ContentGrabBlackList.indexOf(packageName) >= 0) {
            return cloneDocument();
        } else {
            return document.cloneNode(true);
        }
    }

    window.LbDebug = function(msg) {
        window.console.log(msg);
    }

    /**
     * 在判断某个标签是否可见时，要向上寻找父标签是否也是可见
     */
    window.isTagVisible = function(tag){
        while(tag != document.documentElement){
            if (window.getComputedStyle(tag).display == "none" || window.getComputedStyle(tag).opacity == 0) {
                return false;
            }
            tag = tag.parentNode;
        }
        return true;
    }

    /**
     * 判断[x,y]是否在矩形区域内
     */
    window.isInRect = function(left, top, right, bottom, x, y) {
        /*
         只需要确定y坐标是否在矩形区域垂直方向内即可。
         2017.08.24更新：QQ兴趣部落的评论内容是左侧一个图片（img），右边是评论内容文字，当遍历进行按压点所在位置查找时，先找到左侧的图片（img），
         而且垂直方向上满足isInRect的判断，所以，按压点tag就确定是img标签了，导致没有取到文本。所以，isInRect还是要考虑水平和垂直两个方向。
        */
        var inRect = left <= x && right >= x && top <= y && bottom >=y;
        return inRect;
    }

    /**
     * 函数描述：打印Element相关参数，For Debug
     * 实现功能：拼接标签名称、id、className
     */
    window.getElementLog = function(element) {
        return "[tagName: " + element.tagName + ", id: " + element.id + ", className: "
        + element.className + ", position: " + window.getComputedStyle(element).position
        + ", z-index: " + window.getComputedStyle(element)['z-index']
        + ", offsetLeft: " + element.offsetLeft + ", offsetTop: " + element.offsetTop
        +"] ";
    }

    /**
     * 函数描述：获取标签e的绝对坐标
     * 返回值：绝对坐标是否有效，有效返回true，无效返回false
     * 实现功能：通过往父标签遍历直至根标签，将所有父标签的offset值相加，得到左（offsetLeft）上（offsetTop）的绝对坐标。
     */
    window.getElCoordinate = function(e) {
        // 如果是SCRIPT标签 或者 标签不可见，则直接返回isValid为false。
        if (e.tagName == 'SCRIPT' || !isTagVisible(e)) {
            e.coordinate = {
                top: 0,
                left: 0,
                width: 0,
                height: 0,
                bottom: 0,
                right: 0,
                isValid: false
            }
            return e.coordinate.isValid;
        }
        var srcEle = e;

        // 计算元素在页面上的绝对坐标，见http://mixer-b.iteye.com/blog/1563859
        var rect = e.getBoundingClientRect();
        var t = rect.top;
        var l = rect.left;
        var w = rect.width;
        var h = rect.height;

        // 如果找到的同级标签是br这类块标签，offsetTop、offsetLeft以及width、height都是0，则无法计算出坐标；
        // 所以这里暂时将p来替代这类标签，计算出坐标后，再将原标签替换回去。
        if(w == 0) {
            var newEle = document.createElement("p");
            e.parentNode.replaceChild(newEle, e);
            rect = newEle.getBoundingClientRect();
            t = rect.top;
            l = rect.left;
            w = rect.width;
            h = rect.height;
            newEle.parentNode.replaceChild(e, newEle);
        }

        srcEle.coordinate = {
            top: t,
            left: l,
            width: w,
            height: h,
            bottom: t + h,
            right: l + w,
            isValid: w!=0
        }
        return srcEle.coordinate.isValid;
    }

    /**
     * 获取语句sentence中的字数（英文以一个单词作为一个字）
     */
    window.getWordCount = function(sentence) {
        var sentenceLen = sentence.length;
        var words = [];
        var word = "";
        for (var i = 0; i < sentenceLen; i++) {
            var c = sentence.charAt(i);

            // 如果是字母
            if ((c >= 'a' && c <= 'z') || (c >= 'A' && c <= 'Z')) {
                word += c;
            } else {
                if(word.length > 0) {
                    words.push(word);
                    word = "";
                }
                if (c != ' ') {
                    words.push(c);
                }
            }
        }

        // 遍历整个语句之后，如果word不为空，说明语句是以单词结尾的，需要push到words中。
        if(word.length > 0) {
            words.push(word);
            word = "";
        }
        window.console.log("[getWordCount] word count: " + words.length + ", sentence : " + sentence);
        return words.length;
    }

    window.isTimeOut = function(){
        if (typeof(getContentTimeOut) != "undefined") {
            var timestampCur = (new Date()).getTime();
            if (timestampCur - window.HWTimestampStart > getContentTimeOut) {
                return true;
            }
        }
        return false;
    }

})(window)
;

